@@ -32,7 +32,7 @@ import subprocess
 from filecmp import dircmp
 from tempfile import mkdtemp
 
-from .. import bootloader, utils
+from .. import bootloader, timeserver, utils
 from ..config import paths
 from ..lvm import LVM
 from ..naming import Image
@@ -118,7 +118,10 @@ def on_new_layer(imgbase, previous_lv, new_lv):
         threads = []
         threads.append(ThreadRunner(migrate_etc, imgbase, new_lv,
                                     previous_layer_lv))
-        threads.append(ThreadRunner(migrate_root, new_lv, previous_layer_lv))
+        threads.append(ThreadRunner(migrate_state, new_lv, previous_layer_lv,
+                                    "/root/"))
+        threads.append(ThreadRunner(migrate_state, new_lv, previous_layer_lv,
+                                    "/usr/share/rhn/"))
         threads.append(ThreadRunner(relocate_var_lib_yum, new_lv))
 
         thread_group_handler(threads)
@@ -199,6 +202,16 @@ def check_nist_layout(imgbase, new_lv):
                 shutil.copy2(new_config, lvm_config_path)
 
 
+def migrate_state(new_lv, previous_lv, path):
+    log.debug("Migrating %s from the old image to the new image" % path)
+    rsync = Rsync()
+    with mounted(new_lv.path) as new_fs,\
+            mounted(previous_lv.path) as old_fs:
+        old_path = old_fs.path(path)
+        if os.path.isdir(old_path):
+            rsync.sync(old_path, new_fs.path(path))
+
+
 def migrate_var(imgbase, new_lv):
     def strip(s):
         return re.sub(r'^/tmp/mnt.*?/', '', s)
@@ -218,13 +231,6 @@ def migrate_var(imgbase, new_lv):
                         shutil.copy2(newlv_path, realpath)
 
 
-def migrate_root(new_lv, previous_lv):
-    rsync = Rsync()
-    with mounted(new_lv.path) as new_fs,\
-            mounted(previous_lv.path) as old_fs:
-        rsync.sync(old_fs.path("/root/"), new_fs.path("/root"))
-
-
 def boot_partition_validation():
     """
     Function to validate all requirements for /boot partition
@@ -457,6 +463,7 @@ def migrate_etc(imgbase, new_lv, previous_lv):
         log.info("Migrating /root")
 
         threads = []
+        threads.append(ThreadRunner(migrate_ntp_to_chrony, new_lv))
         threads.append(ThreadRunner(run_rpm_perms, new_lv))
         threads.append(ThreadRunner(fix_systemd_services, old_fs, new_fs))
         threads.append(ThreadRunner(run_rpm_selinux_post, new_lv))
@@ -613,6 +620,21 @@ def relocate_var_lib_yum(new_lv):
         os.symlink("/usr/share/yum", "/var/lib/yum")
 
 
+def migrate_ntp_to_chrony(new_lv):
+    with mounted(new_lv.path) as new_fs:
+        if os.path.exists(new_fs.path("/") + "/etc/ntp.conf"):
+            # Create a state directory to track migrations
+            # /var is the right place for application-level data
+            if not os.path.isdir("/var/lib/imgbased"):
+                os.mkdir("/var/lib/imgbased")
+
+            if not os.path.exists("/var/lib/imgbased/ntp-migrated"):
+                log.debug("Migrating NTP configuration to chrony")
+                c = timeserver.Chrony(new_fs.path("/") + "/etc/chrony.conf")
+
+                c.from_ntp(timeserver.Ntp(new_fs.path("/") + "/etc/ntp.conf"))
+
+
 def run_rpm_perms(new_lv):
     with mounted(new_lv.path) as new_fs:
         with utils.bindmounted("/var", new_fs.path("/var"), rbind=True):
